/**
   This class defines Temperature objects.

   I have not given this class a constructor because
   it is not clear how to make the constructor distinguish
   between Celsius and Fahrenheit temperature values.

   You cannot have constructors with names like
      TemperatureC()
   or
      TemperatureF()
   since the name of the constructor must be exactly like
   the name of the class.

   We could have the constructor take a "flag" parameter,
      Temperature(double t, String flag)
   and expect the flsg to hold either "C" or "F", but what
   should the constructor do if the flag gets an improper value?
   We cannot have the constructor return "true" or "false"
   like we did in setTemperatureC() because a constructor
   cannot return a value like a method can.

   We could solve this problem by using something similar to
   a constructor, a "static factory method".
      https://en.wikipedia.org/wiki/Factory_(object-oriented_programming)
*/

public class Temperature
{
   /**
      An instance variable that records a temperature value
      in the degrees Kelvin scale.

      This variable is "private" so that only our code in
      this class file can manipulate it. Our goal is to
      prevent someone from misinterpreting this number.
   */
   private double degreesKelvin;


   /**
      A method for changing the temperature value stored in this object.
      This method assumes that its parameter is a temperature in the
      Celsius scale. If the given temperature value is unrealistic (below
      absolute zero), then it is forced to absolute zero and a value of false
      is returned.
   */
   public boolean setTemperatureC(double degreesC)
   {
      boolean result;
       if (degreesC >= -273)  //validate the data
       {
           degreesKelvin = degreesC + 273;
           result = true;  // report that there was no problem with the parameter
       }
       else
       {
           degreesKelvin = 0;
           result = false;  // report that there was a problem with the parameter
        }
        return result;
   }

   /**
      A method for changing the temperature value stored in this object.
      This method assumes that its parameter is a temperature in the
      Fahrenheit scale. If the given temperature value is unrealistic (below
      absolute zero), then it is forced to absolute zero.
   */
   public void setTemperatureF(double degreesF)
   {
       degreesKelvin = (degreesF - 32)/1.8 + 273;
       //now validate the data
       if ( degreesKelvin < 0 ) degreesKelvin = 0;
   }

   /**
      A method for reading the temperature value stored in this object.
      This method returns a temperature in the Celsius scale.
   */
   public double getTemperatureC()
   {
       return degreesKelvin - 273;
   }

   /**
      A method for reading the temperature value stored in this object.
      This method returns a temperature in the Fahrenheit scale.
   */
   public double getTemperatureF()
   {
       return (degreesKelvin - 273)*1.8 + 32;
   }

   /**
      A method to compare this temperature object with
      another temperature object.
      (See pages 235-239 of the textbook.)
   */
   public boolean equals(Temperature t)
   {
      return ( this.degreesKelvin == t.degreesKelvin );
   }



   /**
      A static factory method to "construct" a Temperature object
      using a Celsius value to initialize the object.
   */
   public static Temperature makeTemperatureC(double degreesC)
   {
      Temperature t = new Temperature();
      t.setTemperatureC(degreesC);
      return t;
   }

   /**
      A static factory method to "construct" a Temperature object
      using a Fahrenheit value to initialize the object.
   */
   public static Temperature makeTemperatureF(double degreesF)
   {
      Temperature t = new Temperature();
      t.setTemperatureF(degreesF);
      return t;
   }

}//Temperature